# pylint: disable=too-many-instance-attributes
class GroupMsg:
    def __init__(self, message: dict):
        self.message = message
        self.CurrentQQ = message.get('CurrentQQ')

        # ========================================
        # 代码提示
        self.FromGroupId = 0
        self.FromGroupName = ''
        self.FromUserId = 0
        self.FromNickName = ''
        self.Content = ''
        self.MsgType = ''
        self.MsgTime = 0
        self.MsgSeq = 0
        self.MsgRandom = 0
        self.RedBaginfo = None
        # ========================================

        temp = message.get('CurrentPacket')
        data = temp.get('Data') if temp is not None else {}

        self.data = data

        self.__init(data)

    def __init(self, data):
        for attr in ['FromGroupId',
                     'FromGroupName',
                     'FromUserId',
                     'FromNickName',
                     'Content',
                     'MsgType',
                     'MsgTime',
                     'MsgSeq',
                     'MsgRandom',
                     'RedBaginfo']:
            setattr(self, attr, data.get(attr))

    def get(self, item):
        return self.message.get(item)

    def __getitem__(self, item):
        return self.message.get(item)

    def __dict__(self, item):
        return self.message.get(item)


class FriendMsg:
    def __init__(self, message: dict):
        self.message = message
        self.CurrentQQ = message.get('CurrentQQ')

        # ========================================
        # 代码提示
        self.FromUin = 0
        self.ToUin = 0
        self.MsgType = ''
        self.MsgSeq = 0
        self.Content = ''
        self.RedBaginfo = None
        # ========================================

        temp = message.get('CurrentPacket')
        data = temp.get('Data') if temp is not None else {}

        self.data = data

        self.__init(data)

    def __init(self, data):
        for attr in ['FromUin',
                     'ToUin',
                     'MsgType',
                     'MsgSeq',
                     'Content',
                     'RedBaginfo']:
            setattr(self, attr, data.get(attr))

    def get(self, item):
        return self.message.get(item)

    def __getitem__(self, item):
        return self.message.get(item)

    def __dict__(self, item):
        return self.message.get(item)


model_map = {
    'OnGroupMsgs': GroupMsg,
    'OnFriendMsgs': FriendMsg,
    # 'OnEvents':EventMsg

}



"""一些常用的方法

其中对于发送语音，图片的方法，建议将timeout设置很短，因为暂时发现这类请求因为需要文件上传操作，
响应时间会较长，而且目前来看，如果文件较大导致上传时间太长，IOTBOT端会报错, IOTBOT响应的结果一定是错误的,
不过发送去的操作是能正常完成的。
"""
import sys

import requests
from requests.exceptions import Timeout

from .client import IOTBOT


class Action:
    def __init__(self,
                 qq='',
                 timeout=10,
                 api_path='/v1/LuaApiCaller',
                 port=8888,
                 host='http://127.0.0.1'):
        self.qq = qq
        self.__timeout = timeout
        self.__api_path = api_path
        self.__port = port
        self.__host = host

    def bind_bot(self, bot: IOTBOT):
        """绑定机器人"""
        self.qq = bot.qq
        self.__port = bot.port
        self.__host = bot.host

    def send_friend_text_msg(self, toUser: int, content: str, timeout=5, **kwargs) -> dict:
        """发送好友文本消息"""
        data = {
            "toUser": toUser,
            "sendToType": 1,
            "sendMsgType": "TextMsg",
            "content": content,
            "groupid": 0,
            "atUser": 0,
            "replayInfo": None
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def get_user_list(self, timeout=5, **kwargs) -> dict:
        """获取好友列表"""
        return self.baseSender('post', 'GetQQUserList', {"StartIndex": 0}, timeout=timeout, **kwargs)

    def send_friend_voice_msg(self, toUser, voiceUrl='', voiceBase64Buf='', timeout=5, **kwargs) -> dict:
        """发送好友语音消息"""
        data = {
            "toUser": toUser,
            "sendToType": 1,
            "sendMsgType": "VoiceMsg",
            "content": "",
            "groupid": 0,
            "atUser": 0,
            "voiceUrl": voiceUrl,
            "voiceBase64Buf": voiceBase64Buf
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_friend_pic_msg(self, toUser, content='', picUrl='', picBase64Buf='', fileMd5='', flashPic=False, timeout=5, **kwargs) -> dict:
        """发送好友图片消息"""
        data = {
            "toUser": toUser,
            "sendToType": 1,
            "sendMsgType": "PicMsg",
            "content": content,
            "groupid": 0,
            "atUser": 0,
            "picUrl": picUrl,
            "picBase64Buf": picBase64Buf,
            "fileMd5": fileMd5,
            "flashPic": flashPic
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_group_text_msg(self, toUser: int, content='', atUser=0, timeout=5, **kwargs) -> dict:
        """发送群文字消息"""
        data = {
            "toUser": toUser,
            "sendToType": 2,
            "sendMsgType": "TextMsg",
            "content": content,
            "groupid": 0,
            "atUser": atUser
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_group_voice_msg(self, toUser, voiceUrl='', voiceBase64Buf='', timeout=5, **kwargs) -> dict:
        """发送群语音"""
        data = {
            "toUser": toUser,
            "sendToType": 2,
            "sendMsgType": "VoiceMsg",
            "content": '',
            "groupid": 0,
            "atUser": 0,
            "voiceUrl": voiceUrl,
            "voiceBase64Buf": voiceBase64Buf
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_group_pic_msg(self, toUser: int, picUrl='', flashPic=False, atUser=0, content='', picBase64Buf='', fileMd5='', timeout=3, **kwargs):
        """发送群图片
        Tips:
            [秀图id] 各id对应效果
            40000   秀图
            40001   幻影
            40002   抖动
            40003   生日
            40004   爱你
            40005   征友
            40006   无(只显示大图无特效)

            [PICFLAG] 改变图文消息顺序
        """
        data = {
            "toUser": toUser,
            "sendToType": 2,
            "sendMsgType": "PicMsg",
            "content": content,
            "groupid": 0,
            "atUser": atUser,
            "picUrl": picUrl,
            "picBase64Buf": picBase64Buf,
            "fileMd5": fileMd5,
            "flashPic": flashPic
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_private_text_msg(self, toUser: int, content: str, groupid: int, timeout=5, **kwargs) -> dict:
        """发送私聊文字消息"""
        data = {
            "toUser": toUser,
            "sendToType": 3,
            "sendMsgType": "TextMsg",
            "content": content,
            "groupid": groupid,
            "atUser": 0
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_private_voice_msg(self, toUser: int, groupid, voiceUrl='', voiceBase64Buf='', timeout=5, **kwargs) -> dict:
        """发送私聊语音"""
        data = {
            "toUser": toUser,
            "sendToType": 3,
            "sendMsgType": "VoiceMsg",
            "content": "",
            "groupid": groupid,
            "atUser": 0,
            "voiceUrl": voiceUrl,
            "voiceBase64Buf": voiceBase64Buf
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_private_pic_msg(self, toUser, groupid, picUrl='', picBase64Buf='', content='', fileMd5='', timeout=5, **kwargs) -> dict:
        """发送私聊图片"""
        data = {
            "toUser": toUser,
            "sendToType": 3,
            "sendMsgType": "PicMsg",
            "content": content,
            "groupid": groupid,
            "atUser": 0,
            "picUrl": picUrl,
            "picBase64Buf": picBase64Buf,
            "fileMd5": fileMd5
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def send_group_json_msg(self, toUser: int, content='', atUser=0, timeout=5, **kwargs) -> dict:
        """发送群Json类型信息"""
        data = {
            "toUser": toUser,
            "sendToType": 2,
            "sendMsgType": "JsonMsg",
            "content": content,
            "groupid": 0,
            "atUser": atUser
        }
        return self.baseSender('POST', 'SendMsg', data, timeout, **kwargs)

    def revoke_msg(self, groupid: int, msgseq: int, msgrandom: int, type_=1, timeout=5, **kwargs) -> dict:
        """撤回消息
        :param type_: 1: RevokeMsg | 2: PbMessageSvc.PbMsgWithDraw
        """
        funcname = 'RevokeMsg' if type_ == 1 else 'PbMessageSvc.PbMsgWithDraw'
        data = {"GroupID": groupid, "MsgSeq": msgseq, "MsgRandom": msgrandom}
        return self.baseSender('POST', funcname, data, timeout, **kwargs)

    def search_group(self, content, page=0, timeout=5, **kwargs) -> dict:
        """搜索群组"""
        return self.baseSender('POST', 'SearchGroup', {"Content": content, "Page": page}, timeout, **kwargs)

    def get_cookies(self, timeout=2, **kwargs) -> dict:
        """获取cookies"""
        return self.baseSender('GET', 'GetUserCook', timeout=timeout, **kwargs)

    def get_group_list(self, timeout=5, **kwargs) -> dict:
        """获取群组列表"""
        return self.baseSender('POST', 'GetGroupList', {"NextToken": ""}, timeout, **kwargs)

    def get_group_user_list(self, groupid: int, timeout=5, **kwargs) -> dict:
        """获取群成员列表"""
        return self.baseSender('POST', 'GetGroupUserList', {"GroupUin": groupid, "LastUin": 0}, timeout, **kwargs)

    def set_unique_title(self, groupid: int, userid: int, Title: str, timeout=1, **kwargs):
        """设置群成员头衔"""
        return self.baseSender('POST', 'OidbSvc.0x8fc_2', {"GroupID": groupid, "UserID": userid, "NewTitle": Title}, timeout, **kwargs)

    def baseSender(self,
                   method: str,
                   funcname: str,
                   data: dict = None,
                   timeout: int = None,
                   api_path: str = None,
                   iot_timeout: int = None,
                   bot_qq: int = None) -> dict:
        """
        :param method: 请求方法
        :param funcname: 请求类型
        :param data: post的数据
        :param timeout: 发送请求等待响应的时间
        :param api_path: 默认为/v1/LuaApiCaller
        :param iot_timeout: IOT端处理请求等待的时间
        :param bot_qq: 机器人QQ
        """

        params = {
            'funcname': funcname,
            'timeout': iot_timeout or self.__timeout,
            'qq': bot_qq or self.qq
        }
        if data is None:
            data = {}
        try:
            rep = requests.request(
                method=method,
                url='{}:{}{}'.format(self.__host, self.__port, api_path or self.__api_path),
                headers={'Content-Type': 'application/json'},
                params=params,
                json=data,
                timeout=timeout or self.__timeout
            )
            rep.raise_for_status()
            if 'Ret' in rep.text and not rep.json()['Ret'] == 0:
                sys.stdout.write(f'请求发送成功, 但处理失败: {rep.json()}')
            return rep.json()
        except Exception as e:
            if isinstance(e, Timeout):
                sys.stdout.write('响应超时，但不代表处理未成功,结果未知!')
                return {}
            sys.stderr.write(f'出现错误: {e}')
            return {}



import sys
import time
from functools import wraps

import socketio

from .model import model_map


def _deco_creater(bind_type):
    def deco(self, func):
        @wraps(func)
        def wrapper(context):
            if not bind_type == 'OnEvents':  # 暂不支持event
                context = model_map[bind_type](context)
            return func(context)
        self.socketio.on(bind_type)(wrapper)
        return wrapper
    return deco


class IOTBOT:
    """
    :param qq: 机器人QQ号
    :param port: 运行端口
    :param beat_delay: 心跳延时时间（s）
    :param host: ip，需要包含协议
    """

    def __init__(self, qq: str, port=8888, beat_delay=60, host='http://127.0.0.1'):
        self.qq = str(qq)
        self.host = host
        self.port = port
        self.beat_delay = beat_delay

        self.__initialize_socketio()

    def run(self):
        try:
            self.socketio.connect(f'{self.host}:{self.port}', transports=['websocket'])
            self.socketio.wait()
        except Exception:
            sys.stdout.write('启动失败，请检查是否已启动IOTBOT。。。')
            sys.exit(1)

    def connect(self):
        print('Connected to server successfully!')
        while True:
            self.socketio.emit('GetWebConn', self.qq)
            time.sleep(self.beat_delay)

    def __initialize_socketio(self):
        self.socketio = socketio.Client()
        self.socketio.event()(self.connect)

    on_group_msg = _deco_creater('OnGroupMsgs')
    on_friend_msg = _deco_creater('OnFriendMsgs')
    on_event = _deco_creater('OnEvents')



"""Author: wongxy github.com/xiyaowong

from iotbot import IOTBOT, GroupMsg, FriendMsg

bot = IOTBOT(your_bot_qq)

@bot.on_group_msg
def group(ctx: GroupMsg):
    pass

@bot.on_friend_msg
def friend(ctx: FriendMsg):
    pass

@bot.on_event
def event(message: dict):
    pass
"""

from .client import IOTBOT
from .model import FriendMsg, GroupMsg, model_map
from .action import Action



import io

from setuptools import setup


def read_files(files):
    data = []
    for file in files:
        with io.open(file, encoding='utf-8') as f:
            data.append(f.read())
    return "\n".join(data)


long_description = read_files(['README.md', 'CHANGELOG.md'])

meta = {}

with io.open('./iotbot/version.py', encoding='utf-8') as f:
    exec(f.read(), meta)

setup(
    name="python-iotbot",
    description="IOTBOT SDK with python!",
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=meta['__version__'],
    author="wongxy",
    author_email="xiyao.wong@foxmail.com",
    url="https://github.com/XiyaoWong/python-iotbot",
    license='MIT',
    keywords=['iotbot', 'iotbot sdk', 'iotqq'],
    packages=['iotbot'],
    install_requires=['python-socketio >= 4.5.1', 'websocket-client >= 0.57.0', 'requests >= 2.23.0'],
    entry_points='''
        [console_scripts]
        iotbot=iotbot.cli:cli
    ''',
    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6'
)



import time

from iotbot import IOTBOT, Action, FriendMsg, GroupMsg

bot_qq = 11
bot = IOTBOT(bot_qq)
action = Action(bot_qq)
# action = Action()
# action.bind_bot(bot)


@bot.on_group_msg
def on_group_msg(ctx: GroupMsg):
    print(ctx.message)
    if ctx.Content == '.test':
        print('CurrentQQ: ', ctx.get('CurrentQQ'))
        print('CurrentPacket: ', ctx.get('CurrentPacket'))
        print('GroupName: ', ctx.FromGroupName)
        print(ctx.MsgType)
        print('发送图片：', action.send_group_pic_msg(ctx.FromGroupId, 'https://t.cn/A6Am7xYO'))
        print('发送闪照：', action.send_group_pic_msg(ctx.FromGroupId, 'https://t.cn/A6Am7xYO', flashPic=True))
        for i in range(1, 3):
            time.sleep(0.5)
            print(action.send_group_pic_msg(ctx.FromGroupId, picUrl='https://t.cn/A6Am7xYO', content=str(i)))
        print(action.send_group_text_msg(ctx.FromGroupId, content='\nOK', atUser=ctx.FromUserId))


@bot.on_friend_msg
def on_friend_msg(ctx: FriendMsg):
    print(ctx.message)
    if ctx.Content == '.test':
        print(ctx.Content)
        print(ctx.FromUin)
        print('OK')
        print('发送闪照：', action.send_friend_pic_msg(ctx.FromUin, 'https://t.cn/A6Am7xYO', flashPic=True))
        action.send_friend_text_msg(ctx.FromUin, '成功')
        action.send_friend_pic_msg(
            ctx.FromUin, picUrl='https://t.cn/A6Am7xYO')


@bot.on_event
def on_event(message: dict):
    # 事件暂时未处理，需要手动操作
    print(message)


if __name__ == "__main__":
    bot.run()



import argparse
import os


def cli():
    parser = argparse.ArgumentParser(description='iotbot模板生成')
    parser.add_argument('-n', default='bot', type=str, help='生成的文件名')
    parser.add_argument('-q', default=12345678, type=int, help='机器人QQ号')

    args = parser.parse_args()
    fileName = args.n
    q = args.q

    template_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'template.py')

    c = input(f'将创建{fileName}.py文件, 机器人QQ为：{q}。是否确定？ y/N: ')
    if c.lower() == 'y':
        with open(template_path, 'r') as f:
            temp = f.read()

        temp = temp.replace('bot_qq = 11', f'bot_qq = {q}')

        with open(f'{fileName}.py', 'w') as f:
            f.write(temp)

        print()
        print('创建成功~')
        print(f"""
执行如下命令：python {fileName}.py

在机器人所在的群或私聊机器人发送：.test
""")
    else:
        print('已取消操作')


if __name__ == "__main__":
    cli()
